==> deepin-upgrade-manager_1.0.16.orig.tar.xz: deepin-upgrade-manager-1.0.16/pkg/module/chroot/chroot.go <==
package chroot

import (
	"deepin-upgrade-manager/pkg/logger"
	"deepin-upgrade-manager/pkg/module/mountpoint"
	"deepin-upgrade-manager/pkg/module/util"
	"errors"
	"os"
	"path/filepath"
	"syscall"
)

type Manager struct {
	target string
	mounts []string
	oldf   *os.File
}

func Start(root string) (Manager, error) {
	logger.Infof("start chroot %s", root)
	var m Manager
	if !util.IsExists(root) {
		return m, errors.New("root dir isn't exit")
	}
	m.target = root
	m.mounts = append(m.mounts, []string{"/proc", "/dev", "/sys"}...)

	err := m.HandleBind()
	if err != nil {
		return m, err
	}
	m.oldf, err = os.Open("/")
	if err != nil {
		return m, err
	}
	return m, syscall.Chroot(m.target)
}

func (m Manager) HandleBind() error {
	for _, v := range m.mounts {
		dst := filepath.Join(m.target, v)
		newInfo := &mountpoint.MountPoint{
			Src:  v,
			Dest: dst,
			Bind: true,
		}
		err := newInfo.Mount()
		if err != nil {
			return nil
		}
	}
	return nil
}

func (m Manager) HandleUnBind() error {
	for _, v := range m.mounts {
		dst := filepath.Join(m.target, v)
		newInfo := &mountpoint.MountPoint{
			Src:  v,
			Dest: dst,
			Bind: true,
		}
		err := newInfo.Umount()
		if err != nil {
			return nil
		}
	}
	return nil
}

func (m Manager) Exit() error {
	logger.Infof("exit chroot %s", m.target)
	defer func() {
		if m.oldf != nil {
			m.oldf.Close()
		}
	}()
	err := m.oldf.Chdir()
	if err != nil {
		return err
	}
	err = syscall.Chroot(".")
	if err != nil {
		return err
	}
	return m.HandleUnBind()
}

==> deepin-upgrade-manager_1.0.17.orig.tar.xz: deepin-upgrade-manager-1.0.17/pkg/module/chroot/chroot.go <==
package chroot

import (
	"deepin-upgrade-manager/pkg/logger"
	"deepin-upgrade-manager/pkg/module/dirinfo"
	"deepin-upgrade-manager/pkg/module/mountpoint"
	"deepin-upgrade-manager/pkg/module/util"
	"errors"
	"os"
	"path/filepath"
	"syscall"
)

type Manager struct {
	target string
	mounts []string
	oldf   *os.File
}

func Start(root string) (Manager, error) {
	logger.Infof("start chroot %s", root)
	var m Manager
	if !util.IsExists(root) {
		return m, errors.New("root dir isn't exit")
	}
	m.target = root
	m.mounts = append(m.mounts, []string{"/proc", "/dev", "/sys", "/run"}...)

	err := m.HandleBind()
	if err != nil {
		return m, err
	}
	m.oldf, err = os.Open("/")
	if err != nil {
		return m, err
	}
	return m, syscall.Chroot(m.target)
}

func (m Manager) HandleBind() error {
	for _, v := range m.mounts {
		dst := filepath.Join(m.target, v)
		newInfo := &mountpoint.MountPoint{
			Src:  v,
			Dest: dst,
			Bind: true,
		}
		err := newInfo.Mount()
		if err != nil {
			return nil
		}
	}
	return nil
}

func (m Manager) HandleUnBind() error {
	for _, v := range m.mounts {
		dst := filepath.Join(m.target, v)
		newInfo := &mountpoint.MountPoint{
			Src:  v,
			Dest: dst,
			Bind: true,
		}
		err := newInfo.Umount()
		if err != nil {
			return nil
		}
	}
	return nil
}

func (m Manager) Exit() error {
	logger.Infof("exit chroot %s", m.target)
	defer func() {
		if m.oldf != nil {
			m.oldf.Close()
		}
	}()
	err := m.oldf.Chdir()
	if err != nil {
		return err
	}
	err = syscall.Chroot(".")
	if err != nil {
		return err
	}
	return m.HandleUnBind()
}

func IsEnv() bool {
	part, err := dirinfo.GetDirPartition("/")
	if err != nil {
		return true
	}
	if !util.IsExists(part) {
		return true
	}
	return false
}
